@@ -41,6 +41,12 @@
 typedef struct lnf_ip_s { uint32_t data[4]; } lnf_ip_t; /*!< IPv4/IPv6 address */
 typedef struct lnf_mac_s { uint8_t data[6]; }  lnf_mac_t; /*!< MAC address */
 typedef struct lnf_mpls_s { uint32_t data[10]; } lnf_mpls_t; /*!< MPLS tags */
+typedef struct lnf_acl_s {  /*!< ACL ID (http://www.cisco.com/c/en/us/td/docs/security/asa/asa93/netflow/netflow.html) */
+	uint32_t acl_id;
+	uint32_t ace_id;
+	uint32_t xace_id;
+} lnf_acl_t; 
+
 
 /* basic record type 1 - contains the most commonly used fields */
 typedef struct lnf_brec1_s {
@@ -63,16 +69,29 @@ typedef struct lnf_brec1_s {
 /* note: if the fields type allows two kind of data type  */
 /* for example UINT32 and UINT64 libnf always uses the biggest one */
 #define LNF_NONE			0x00
+#define LNF_NONE_T			void
 #define LNF_UINT8			0x08
+#define LNF_UINT8_T			uint8_t
 #define LNF_UINT16			0x16
+#define LNF_UINT16_T		uint16_t
 #define LNF_UINT32			0x32
+#define LNF_UINT32_T		uint32_t
 #define LNF_UINT64			0x64
+#define LNF_UINT64_T		uint64_t
 #define LNF_DOUBLE			0x70
+#define LNF_DOUBLE_T		double
 #define LNF_ADDR 			0xA1	/* 128 bit addr (struct in6_addr/network order) */
+#define LNF_ADDR_T 			lnf_ip_t
 #define LNF_MAC				0xA2
-#define LNF_STRING			0xAA	/* null terminated string */
+#define LNF_MAC_T			lnf_mac_t
+#define LNF_STRING			0xAA	/* null terminated string, max 255 characters */
+#define LNF_STRING_T		char[256]
 #define LNF_MPLS			0xAB	/* mpls labels */
+#define LNF_MPLS_T			lnf_mpls_t
+#define LNF_ACL				0xAC	/* ACL  */
+#define LNF_ACL_T			lnf_acl_t
 #define LNF_BASIC_RECORD1	0xB1
+#define LNF_BASIC_RECORD1_T	lnf_brec1_t
 
 
 #define LNF_MASK_TYPE  		0x0000FF
@@ -152,6 +171,8 @@ typedef struct lnf_brec1_s {
 #define LNF_FLD_SERVER_NW_DELAY_USEC	0x3f 
 #define LNF_FLD_APPL_LATENCY_USEC		0x40 
 #define LNF_FLD_FW_EVENT		0x41
+#define LNF_FLD_INGRESS_ACL		0x42
+#define LNF_FLD_EGRESS_ACL		0x43
 
 /* computed and extra fields */
 #define LNF_FLD_CALC_DURATION	 0xA0 			/* computed : duration in msec  */
@@ -196,11 +217,12 @@ typedef void lnf_mem_cursor_t;
 #define LNF_OK				0x0001	/* OK status */
 #define LNF_EOF 			0x0000	/* end of file */
 
-#define LNF_READ			0x0
-#define LNF_WRITE			0x1
-#define LNF_ANON			0x2
-#define LNF_COMP			0x4
-#define LNF_WEAKERR			0x8
+#define LNF_READ			0x00
+#define LNF_WRITE			0x01
+#define LNF_APPEND			0x10
+#define LNF_ANON			0x02
+#define LNF_COMP			0x04
+#define LNF_WEAKERR			0x08
 
 #define LNF_ERR_UNKBLOCK	-0x0001	/* weak error: unknown block type */
 #define LNF_ERR_UNKREC		-0x0002	/* weak error: unknown record type */
@@ -219,6 +241,7 @@ typedef void lnf_mem_cursor_t;
 #define LNF_ERR_NOMEM		-0x0800	/* cannot allocate memory  */
 #define LNF_ERR_OTHER		-0x0F00	/* some other error */
 #define LNF_ERR_OTHER_MSG	-0x1000	/* other error and more descriptive information can be get from lnf_error */
+#define LNF_ERR_NAN			-0x2000	/* attempt to divide by 0 */
 
 
 /* constants for lnf_info function */
@@ -283,6 +306,7 @@ structure (see record operations).
 \param *filename 	path and file name to open 
 \param flags 		flags, described above \n
 	LNF_READ - open file for reading  \n
+	LNF_APPEND - open file for reading in append mode \n
 	LNF_WRITE - open file for for writing  \n
 	LNF_ANON - set anon flag on the file (only for write mode) \n
 	LNF_COMP - set the output file to be compressed \n
@@ -523,6 +547,7 @@ int lnf_mem_init(lnf_mem_t **lnf_mem);
 /* options for lnf_mem_t */
 #define LNF_OPT_HASHBUCKETS	0x0001	/* number of buckets in hash  */
 #define LNF_OPT_LISTMODE	0x0002	/* switch lnf_mem_t to list mode */
+#define LNF_OPT_COMP_STATSCMP	0x0003	/* switch lnf_mem_t compatible statistics mode */
 
 /*!	\ingroup memheap
 \brief Set lnf_mem_t options 
@@ -537,6 +562,35 @@ sorting items without aggregation.
 \param info 		option to set - value of option is stored in *data\n
 	LNF_OPT_HASHBUCKETS - the number of buckets in hash table (int) (default 500000)
 	LNF_OPT_LISTMODE - switch lnf_mem into linked list mode (NULL)
+	LNF_OPT_COMP_STATSCMP - switch lnf_mem into mode where pair statistics compatible mode with nfdump 
+		
+In nfdump when the statistics i computed for the part item (for example port) the record is counted 
+twice - first for src port and then for destination port. However if the source and 
+destination port is the same nfdump counts the record only once. By default libnf always 
+counts the records with pair fields twice. This option switches libnf to the nfdump behavior. 
+
+Example: 
+We have input flow 
+SRC           DST          PKTS BYTES
+1.1.1.1:53 -> 2.2.2.2:53      1   20 
+3.3.3.3:80 -> 4.4.4.4:1222    3   80
+
+and we have statistics via port field 
+
+In the nfdump and libnf with LNF_OPT_COMP_STATSCMP option enabled 
+the result will be:
+PORT    PKTS BYTES
+53         1    20
+80         3    80
+1222       3    80
+
+but in libnf  without LNF_OPT_COMP_STATSCMP option the result will be: 
+PORT    PKTS BYTES
+53         2    40
+80         3    80
+1222       3    80
+
+
 \param *data 		pointer to data structure
 \param size			data size
 \return 			LNF_OK, LNF_ERR_OTHER
@@ -753,10 +807,17 @@ void lnf_mem_free(lnf_mem_t *lnf_mem);
 int lnf_fld_type(int field);
 #define LNF_FLD_INFO_FIELDS	0x01	/* fill array of ints ended with LNF_FLD_TERM_  */
 #define LNF_FLD_INFO_TYPE	0x02	/* type - int */
+#define LNF_FLD_INFO_SIZE	0x1F	/* field size in Bytes, 0 - for dynamics size fields */
 #define LNF_FLD_INFO_NAME	0x04	/* name - char* */
 #define LNF_FLD_INFO_DESCR	0x08	/* description - char * */
 #define LNF_FLD_INFO_AGGR	0x0B	/* default aggregation - int */
 #define LNF_FLD_INFO_SORT	0x0E	/* default sort - int */
+#define LNF_FLD_INFO_IPFIX_NAME		0x11	/* ipfix name  - char */
+#define LNF_FLD_INFO_IPFIX_EID		0x12	/* enterprise ID - int */
+#define LNF_FLD_INFO_IPFIX_ID		0x14	/* element ID - int */
+#define LNF_FLD_INFO_IPFIX_NAME6	0x18	/* ipfix name - char */
+#define LNF_FLD_INFO_IPFIX_EID6		0x1B	/* enterprise ID - int */
+#define LNF_FLD_INFO_IPFIX_ID6		0x1E	/* element ID - int */
 
 #define LNF_INFO_BUFSIZE 4096		/* maximum buffer size for data lnf_*_fields */
 /* return LNF_OK or LNF_ERR_UNKFLD or LNF_ERR_OTHER */
