@@ -15,7 +15,7 @@ use threads;
 
 our @ISA = qw(Exporter);
 
-our $VERSION = '1.16';
+our $VERSION = '1.17';
 
 # XXX
 # Items to export into callers namespace by default. Note: do not export
@@ -72,6 +72,13 @@ XSLoader::load('Net::NfDump', $VERSION);
 
 # Below is stub documentation for your module. You'd better edit it!
 
+# how to convert particular type to 
+my %CVTTYPE = ( 
+	'ip' => 'ip', 'srcip' => 'ip', 'dstip' => 'ip', 'nexthop' => 'ip', 'bgpnexthop' => 'ip', 'router' => 'ip',
+	'insrcmac' => 'mac', 'outsrcmac' => 'mac', 'indstmac' => 'mac', 'outdstmac' => 'mac',
+	'mpls' => 'mpls',
+	'xsrcip' => 'ip', 'xdstip' => 'ip', 'nsrcip' => 'ip', 'ndstip' => 'ip' );
+
 =head1 NAME
 
 Net::NfDump - Perl API for manipulating with nfdump files based on libnf.net library 
@@ -349,6 +356,12 @@ Flag indicating that output file contains anonymized data.
 
 String identificator of files. The value is stored in the file header. 
 
+=item * B<CompatMode> => 0
+
+Enable nfdump compatibility features. Some features are implemented differently 
+comparing to original nfdump. Currently thi option enables only 
+LNF_OPT_COMP_STATSCMP for aggregated statistics computation. 
+
 =back 
 
 =head2 Constructor, status information methods
@@ -524,13 +537,17 @@ sub query {
 	my $o = $self->merge_opts(%opts);
 
 	if (@{$o->{InputFiles}} == 0) {
-		croak("No imput files defined");
+		croak("No input files defined");
 	} 
 
 	my @resfields = ();
 	foreach my $fld (@{$o->{Fields}}) {
-		my $numbits = 32;
-		my $numbits6 = 128;
+		my $numbits = 0;
+		my $numbits6 = 0;
+		if (defined($CVTTYPE{$fld}) && $CVTTYPE{$fld} eq 'ip') {
+			$numbits = 32;
+			$numbits6 = 128;
+		} 
 
 		if ($fld =~ /\//) {
 			($fld, $numbits, $numbits6) = split(/\//, $fld);
@@ -541,7 +558,7 @@ sub query {
 
 		if (defined($o->{Aggreg}) && $o->{Aggreg}) {
 			if ($fld eq '*') {
-				croak("Symbol '*' is not allower for aggregated items");
+				croak("Symbol '*' is not allowed for aggregated items");
 			}
 		
 			if (!defined($Net::NfDump::Fields::NFL_FIELDS_TXT{$fld})) {
@@ -549,7 +566,8 @@ sub query {
 			}
 
 			my $id = $Net::NfDump::Fields::NFL_FIELDS_TXT{$fld};
-			my $flags = $Net::NfDump::Fields::NFL_FIELDS_DEFAULT_AGGR{$id};
+#			my $flags = $Net::NfDump::Fields::NFL_FIELDS_DEFAULT_AGGR{$id};
+			my $flags = 0;
 
 			if (defined($o->{OrderBy}) && $fld eq $o->{OrderBy}) {
 				$flags |= $Net::NfDump::Fields::NFL_FIELDS_DEFAULT_SORT{$id};
@@ -572,6 +590,11 @@ sub query {
 
 	$self->set_fields([ @resfields ]);
 
+	if (defined($o->{CompatMode}) && $o->{CompatMode}) {
+		Net::NfDump::libnf_compatmode($self->{handle});
+	}
+
+
 	# handle, filter, windows start, windows end, ref to filelist 
 	Net::NfDump::libnf_read_files($self->{handle}, $o->{Filter}, 
 				$o->{TimeWindowStart}, $o->{TimeWindowEnd}, 
@@ -857,7 +880,7 @@ sub ip2txt ($) {
 	} elsif (length($addr) == 16) {
 		$type = AF_INET6;
 	} else {
-		carp("Invalid IP address length in binnary representation");
+		carp("Invalid IP address length in binary representation");
 		return undef;
 	}
 
@@ -913,7 +936,7 @@ sub mac2txt ($) {
 	}
 
 	if (length($addr) != 6) {
-		carp("Invalid MAC address length in binnary representation");
+		carp("Invalid MAC address length in binary representation");
 		return undef;
 	}
 
@@ -1017,12 +1040,6 @@ txt2flow does the exact opossite.
 
 =cut 
 
-# how to convert particular type to 
-my %CVTTYPE = ( 
-	'ip' => 'ip', 'srcip' => 'ip', 'dstip' => 'ip', 'nexthop' => 'ip', 'bgpnexthop' => 'ip', 'router' => 'ip',
-	'insrcmac' => 'mac', 'outsrcmac' => 'mac', 'indstmac' => 'mac', 'outdstmac' => 'mac',
-	'mpls' => 'mpls',
-	'xsrcip' => 'ip', 'xdstip' => 'ip', 'nsrcip' => 'ip', 'ndstip' => 'ip' );
 	
 
 sub flow2txt ($) {
