@@ -281,24 +281,26 @@ int lnf_open(lnf_file_t **lnf_filep, const char * filename, unsigned int flags,
 
 	lnf_file->flags = flags;
 	/* open file in either read only or write only mode */
-	if (flags & LNF_WRITE) {
 #ifdef LNF_THREADS
-    	pthread_mutex_lock(&lnf_nfdump_filter_mutex);
+   	pthread_mutex_lock(&lnf_nfdump_filter_mutex);
 #endif
+	if (flags & LNF_APPEND) {
+
+		lnf_file->nffile = AppendFile((char *)filename);
+
+	} else if (flags & LNF_WRITE) {
+
 		lnf_file->nffile = OpenNewFile((char *)filename, NULL, flags & LNF_COMP, 
 								flags & LNF_ANON, (char *)ident);
-#ifdef LNF_THREADS
-    	pthread_mutex_unlock(&lnf_nfdump_filter_mutex);
-#endif
+
 	} else {
-#ifdef LNF_THREADS
-    	pthread_mutex_lock(&lnf_nfdump_filter_mutex);
-#endif
+
 		lnf_file->nffile = OpenFile((char *)filename, NULL);
-#ifdef LNF_THREADS
-    	pthread_mutex_unlock(&lnf_nfdump_filter_mutex);
-#endif
+
 	}
+#ifdef LNF_THREADS
+   	pthread_mutex_unlock(&lnf_nfdump_filter_mutex);
+#endif
 
 	if (lnf_file->nffile == NULL) {
 		free(lnf_file);
@@ -749,24 +751,31 @@ int lnf_rec_init(lnf_rec_t **recp) {
 	rec = malloc(sizeof(lnf_rec_t)); 
 
 	if (rec == NULL) {
+		*recp = NULL;
 		return LNF_ERR_NOMEM;
 	}
 
+	rec->extensions_arr = NULL;
+	rec->field_data = NULL;
+
 	rec->master_record = malloc(sizeof(master_record_t));
 
 	if (rec->master_record == NULL) {
-		free(rec);
+		lnf_rec_free(rec);
+		*recp = NULL;
 		return LNF_ERR_NOMEM;
 	}
 
 	rec->extensions_arr = malloc(sizeof(bit_array_t));
 
 	if (rec->extensions_arr == NULL) {
-		free(rec->master_record);
-		free(rec);
+		lnf_rec_free(rec);
+		*recp = NULL;
 		return LNF_ERR_NOMEM;
 	}
 
+
+	/* initialise nfdump extension list */
 	i = 1;
 	numext = 0;
 	while ( extension_descriptor[i++].id ) {
@@ -774,12 +783,32 @@ int lnf_rec_init(lnf_rec_t **recp) {
 	}
 
 	if (!bit_array_init(rec->extensions_arr, numext + 1)) {
-		free(rec->extensions_arr);
-		free(rec->master_record);
-		free(rec);
+		lnf_rec_free(rec);
+		*recp = NULL;
 		return LNF_ERR_NOMEM;
 	}
 
+	/* initialise (non nfdump) field data */
+	rec->field_data = malloc( LNF_FLD_TERM_ * sizeof(void*) );
+	if (rec->field_data == NULL) {
+		lnf_rec_free(rec);
+		*recp = NULL;
+		return LNF_ERR_NOMEM;
+	}
+
+	memset(rec->field_data, 0x0, LNF_FLD_TERM_ * sizeof(void*));
+
+	for (i = LNF_FLD_ZERO_; i < LNF_FLD_TERM_; i++) {
+		if (lnf_fields_def[i].type !=  LNF_FLD_ZERO_) {
+			rec->field_data[i] = malloc(lnf_fields_def[i].size);
+			if (rec->field_data[i] == NULL) { 
+				lnf_rec_free(rec);
+				*recp = NULL;
+				return LNF_ERR_NOMEM;
+			} 
+		}
+	}
+
 	lnf_rec_clear(rec);
 
 	*recp = rec; 
@@ -814,13 +843,28 @@ int lnf_rec_copy(lnf_rec_t *dst, lnf_rec_t *src) {
 /* free record */
 void lnf_rec_free(lnf_rec_t *rec) {
 
+	int i;
+
 	if (rec == NULL) {
 		return;
 	}
 
-	bit_array_release(rec->extensions_arr);
-	free(rec->master_record);
-	free(rec->extensions_arr);
+	if (rec->extensions_arr != NULL) {
+		bit_array_release(rec->extensions_arr);
+		free(rec->extensions_arr);
+	}
+
+	if (rec->master_record != NULL) {
+		free(rec->master_record);
+	}
+
+	if (rec->field_data != NULL) {
+		for (i = LNF_FLD_ZERO_; i < LNF_FLD_TERM_; i++) {
+			if (rec->field_data[i] != NULL) {
+				free(rec->field_data[i]);
+			}
+		}
+	}
 	free(rec);
 }
 
