@@ -89,6 +89,9 @@ typedef struct lnf_filter_s {
 typedef struct lnf_rec_s {
 	master_record_t *master_record;		/* reference to master record */
 	bit_array_t *extensions_arr;		/* list of extensions available in the record */
+	void **field_data;					/* list of pointers to data field */
+										/* data field is represented by pointer */
+										/* field_data[field] */
 } lnf_rec_t;
 
 
@@ -182,7 +185,9 @@ typedef struct lnf_mem_s {
 	int sorted;						/* is the table sorted ? */
 	int statistics_mode;			/* is lnf_mem in statistics mode ? (have a pair item in kay) */
 	int list_mode;					/* is lnf_mem in linked list mode */
+	int nfdump_comp_statistics;		/* is lnf_mem nfdump compatible statistics mode */
 	int hash_table_buckets;			/* initial number of buckets in hash table */
+	lnf_rec_t	*lnf_rec;			/* temporary lnf_rec entry */
 } lnf_mem_t;
 
 
@@ -194,9 +199,11 @@ void lnf_filedlist_free(lnf_fieldlist_t *list);
 void lnf_clear_bits(char *buf, int buflen, int from);
 int lnf_mem_fill_buf(lnf_fieldlist_t *fld, lnf_rec_t *rec, char *buf, int pairset);
 void lnf_mem_fill_rec(lnf_fieldlist_t *fld, char *buf, lnf_rec_t *rec);
+void lnf_mem_upd_calc_fields(lnf_mem_t *lnf_mem);
 
 
 int lnf_filedlist_add_or_upd(lnf_fieldlist_t **list, lnf_fieldlist_t *snode, int *sizep, int maxsize, int *offset);
+lnf_fieldlist_t* lnf_filedlist_lookup(lnf_fieldlist_t *list, int field);
 void lnf_mem_fastaggr_callback(char *key, char *hval, char *uval, void *lnf_mem);
 void lnf_mem_aggr_callback(char *key, char *hval, char *uval, void *lnf_mem);
 int lnf_mem_sort_callback(char *key1, char *val1, char *key2, char *val2, void *p);
