@@ -54,6 +54,7 @@ GEN_AGGR_SUM(lnf_mem_aggr_SUM_UINT8, uint8_t);
 GEN_AGGR_SUM(lnf_mem_aggr_SUM_UINT16, uint16_t);
 GEN_AGGR_SUM(lnf_mem_aggr_SUM_UINT32, uint32_t);
 GEN_AGGR_SUM(lnf_mem_aggr_SUM_UINT64, uint64_t);
+GEN_AGGR_SUM(lnf_mem_aggr_SUM_DOUBLE, double);
 
 
 #define GEN_AGGR_OR(name, type) static void name (char *a, char *b) { \
@@ -63,6 +64,7 @@ GEN_AGGR_OR(lnf_mem_aggr_OR_UINT8, uint8_t);
 GEN_AGGR_OR(lnf_mem_aggr_OR_UINT16, uint16_t);
 GEN_AGGR_OR(lnf_mem_aggr_OR_UINT32, uint32_t);
 GEN_AGGR_OR(lnf_mem_aggr_OR_UINT64, uint64_t);
+//GEN_AGGR_OR(lnf_mem_aggr_OR_DOUBLE, double);
 								
 #define GEN_AGGR_MIN(name, type) static void name (char *a, char *b) {	\
 	if ( *((type *)a) > *((type *)b) ) *((type *)a) = *((type *)b); \
@@ -71,6 +73,7 @@ GEN_AGGR_MIN(lnf_mem_aggr_MIN_UINT8, uint8_t);
 GEN_AGGR_MIN(lnf_mem_aggr_MIN_UINT16, uint16_t);
 GEN_AGGR_MIN(lnf_mem_aggr_MIN_UINT32, uint32_t);
 GEN_AGGR_MIN(lnf_mem_aggr_MIN_UINT64, uint64_t);
+GEN_AGGR_MIN(lnf_mem_aggr_MIN_DOUBLE, double);
 
 #define GEN_AGGR_MAX(name, type) static void name (char *a, char *b) {	\
 	if ( *((type *)a) < *((type *)b) ) *((type *)a) = *((type *)b); \
@@ -79,6 +82,7 @@ GEN_AGGR_MAX(lnf_mem_aggr_MAX_UINT8, uint8_t);
 GEN_AGGR_MAX(lnf_mem_aggr_MAX_UINT16, uint16_t);
 GEN_AGGR_MAX(lnf_mem_aggr_MAX_UINT32, uint32_t);
 GEN_AGGR_MAX(lnf_mem_aggr_MAX_UINT64, uint64_t);
+GEN_AGGR_MAX(lnf_mem_aggr_MAX_DOUBLE, double);
 
 static void lnf_mem_aggr_EMPTY (char *a, char *b) { }
 
@@ -105,12 +109,20 @@ int lnf_mem_init(lnf_mem_t **lnf_memp) {
 	lnf_mem->fastaggr_mode = LNF_FAST_AGGR_NONE;
 
 	//lnf_mem->hash_ptr = NULL;
+	lnf_mem->sort_field = LNF_FLD_ZERO_; 
 	lnf_mem->sorted = 0;
 	lnf_mem->statistics_mode = 0;
 	lnf_mem->list_mode = 0;
+	lnf_mem->nfdump_comp_statistics = 0;
 	lnf_mem->numthreads = 0;
 	lnf_mem->read_cursor = NULL;
 	lnf_mem->hash_table_buckets = HASH_TABLE_INIT_SIZE;
+	
+	if (!lnf_rec_init(&lnf_mem->lnf_rec)) {
+		free(lnf_mem);
+		return LNF_ERR_NOMEM;
+	}
+	
 #ifdef LNF_THREADS
 	if (pthread_mutex_init(&lnf_mem->thread_mutex, NULL) != 0) {
 		free(lnf_mem);
@@ -125,7 +137,6 @@ int lnf_mem_init(lnf_mem_t **lnf_memp) {
 	lnf_mem->thread_id_key = NULL;
 #endif
 
-
 	/* set all threads to LNF_TH_EMPTY */
 	for (i = 0; i < LNF_MAX_THREADS; i++) {
 		lnf_mem->thread_status[i] = LNF_TH_EMPTY;
@@ -201,6 +212,9 @@ int lnf_mem_setopt(lnf_mem_t *lnf_mem, int opt, void *data, size_t size) {
 			lnf_mem->list_mode = 1;
 			break;
 
+		case LNF_OPT_COMP_STATSCMP:
+			lnf_mem->nfdump_comp_statistics = 1;
+			break;
 		default: 
 			return LNF_ERR_OTHER;
 			break;
@@ -281,6 +295,26 @@ int lnf_filedlist_add_or_upd(lnf_fieldlist_t **list, lnf_fieldlist_t *snode, int
 	return LNF_OK;
 }
 
+/*! 
+* lookup field in linked list and retur pointer to node or NULL
+*/
+lnf_fieldlist_t* lnf_filedlist_lookup(lnf_fieldlist_t *list, int field) {
+
+	lnf_fieldlist_t *tmp_node;
+
+	/* find item in list and update */
+	tmp_node = list;
+	while (tmp_node != NULL) {
+		if (tmp_node->field == field) {
+			return tmp_node;
+		} else {
+			tmp_node = tmp_node->next;
+		}
+	}
+
+	return NULL;
+}
+
 void lnf_filedlist_free(lnf_fieldlist_t *list) {
 
 	lnf_fieldlist_t *node, *tmp_node;
@@ -317,7 +351,7 @@ int lnf_mem_fastaggr(lnf_mem_t *lnf_mem, int mode) {
 int lnf_mem_fadd(lnf_mem_t *lnf_mem, int field, int flags, int numbits, int numbits6) {
 
 	lnf_fieldlist_t fld;
-	int offset;
+	int offset, calcnum, ret;
 	
 	fld.field = field;
 	switch (lnf_fld_type(field)) { 
@@ -325,6 +359,7 @@ int lnf_mem_fadd(lnf_mem_t *lnf_mem, int field, int flags, int numbits, int numb
 		case LNF_UINT16: fld.size = sizeof(uint16_t); break;
 		case LNF_UINT32: fld.size = sizeof(uint32_t); break;
 		case LNF_UINT64: fld.size = sizeof(uint64_t); break;
+		case LNF_DOUBLE: fld.size = sizeof(LNF_DOUBLE_T); break;
 		case LNF_ADDR: fld.size = sizeof(lnf_ip_t); break;
 		case LNF_MAC: fld.size = sizeof(lnf_mac_t); break;
 		default : 
@@ -334,13 +369,19 @@ int lnf_mem_fadd(lnf_mem_t *lnf_mem, int field, int flags, int numbits, int numb
 	fld.type = lnf_fld_type(field);
 	fld.numbits = numbits;
 	fld.numbits6 = numbits6;
-	if (!flags) {
-		lnf_fld_info(field, LNF_FLD_INFO_AGGR, &fld.aggr_flag, sizeof(fld.aggr_flag));
+	if ((flags & LNF_AGGR_FLAGS) == 0) {
+		/* if type is UINT64 and numbits is set then field is key */
+		if (fld.numbits > 0 && fld.type == LNF_UINT64) {
+			fld.aggr_flag = LNF_AGGR_KEY;
+		} else {
+			lnf_fld_info(field, LNF_FLD_INFO_AGGR, &fld.aggr_flag, sizeof(fld.aggr_flag));
+		}
 	} else {
 		fld.aggr_flag = flags & LNF_AGGR_FLAGS;
-		fld.sort_flag = flags & LNF_SORT_FLAGS;
 	}
 
+	fld.sort_flag = flags & LNF_SORT_FLAGS;
+		
 
 	/* select aggregation func for item */
 	fld.aggr_func = lnf_mem_aggr_EMPTY;
@@ -373,19 +414,26 @@ int lnf_mem_fadd(lnf_mem_t *lnf_mem, int field, int flags, int numbits, int numb
 			case LNF_AGGR_MAX: fld.aggr_func = lnf_mem_aggr_MAX_UINT64; break;
 			case LNF_AGGR_OR:  fld.aggr_func = lnf_mem_aggr_OR_UINT64; break;
 			} break;
+		case LNF_DOUBLE: 
+			switch (fld.aggr_flag) {
+			case LNF_AGGR_SUM: fld.aggr_func = lnf_mem_aggr_SUM_DOUBLE; break;
+			case LNF_AGGR_MIN: fld.aggr_func = lnf_mem_aggr_MIN_DOUBLE; break;
+			case LNF_AGGR_MAX: fld.aggr_func = lnf_mem_aggr_MAX_DOUBLE; break;
+			case LNF_AGGR_OR:  fld.aggr_func = lnf_mem_aggr_EMPTY; break;
+			} break;
 		/* other data types are ignored so far */
 	}
 	
 
 	/* add to key list */
-	if ((flags & LNF_AGGR_FLAGS) == LNF_AGGR_KEY) {
+	if (fld.aggr_flag == LNF_AGGR_KEY) {
 		if ( lnf_filedlist_add_or_upd(&lnf_mem->key_list, &fld, &lnf_mem->key_len, LNF_MAX_KEY_LEN, &offset) != LNF_OK ) {
 			return LNF_ERR_NOMEM;
 		}
-		if ((flags & LNF_SORT_FLAGS) != LNF_SORT_NONE) {
+		if (fld.sort_flag != LNF_SORT_NONE) {
 			lnf_mem->sort_field = field;
 			lnf_mem->sort_offset = offset;
-			lnf_mem->sort_flags = LNF_SORT_FLD_IN_KEY | (flags & LNF_SORT_FLAGS);
+			lnf_mem->sort_flags = LNF_SORT_FLD_IN_KEY | fld.sort_flag;
 		}
 		/* ser statistics mode for lnf_mem if there is pair field */
 		if (__lnf_fld_pair(field, 1) != LNF_FLD_ZERO_ && __lnf_fld_pair(field, 2) != LNF_FLD_ZERO_) {
@@ -395,13 +443,30 @@ int lnf_mem_fadd(lnf_mem_t *lnf_mem, int field, int flags, int numbits, int numb
 		if ( lnf_filedlist_add_or_upd(&lnf_mem->val_list, &fld, &lnf_mem->val_len, LNF_MAX_VAL_LEN, &offset) != LNF_OK ) {
 			return LNF_ERR_NOMEM;
 		}
-		if ((flags & LNF_SORT_FLAGS) != LNF_SORT_NONE) {
+		if (fld.sort_flag != LNF_SORT_NONE) {
 			lnf_mem->sort_field = field;
 			lnf_mem->sort_offset = offset;
-			lnf_mem->sort_flags = LNF_SORT_FLD_IN_VAL | (flags & LNF_SORT_FLAGS);
+			lnf_mem->sort_flags = LNF_SORT_FLD_IN_VAL | fld.sort_flag;
 		}
 	}
 
+	/* if the filed is calculated field with dependencies add filds that the firld is depenedend on */
+	calcnum = 0;
+	while ( __lnf_fld_calc_dep(field, calcnum) != LNF_FLD_ZERO_) {
+
+		if ( (lnf_filedlist_lookup(lnf_mem->key_list, __lnf_fld_calc_dep(field, calcnum)) == NULL) &&
+			 (lnf_filedlist_lookup(lnf_mem->val_list, __lnf_fld_calc_dep(field, calcnum)) == NULL) ) {
+
+			if ((ret = lnf_mem_fadd(lnf_mem, __lnf_fld_calc_dep(field, calcnum), 0, 0,  0)) != LNF_OK) {
+				return ret;
+			}
+
+		}
+
+		calcnum++;
+
+	}
+
 	return LNF_OK;
 }
 
@@ -417,6 +482,16 @@ static void inline lnf_clear_bits_v4(uint32_t *buf, int from) {
 
 }
 
+
+/* align time value */
+static void lnf_align_uint64(uint64_t *buf, int align) {
+
+	if (align > 0) {
+		*buf = *buf - (*buf % (align * 1000));	
+	}
+
+}
+
 /* set bits "from" to zero value (for IPv6) */
 static void inline lnf_clear_bits_v6(uint64_t *buf, int from) {
 	uint64_t tmp;
@@ -447,7 +522,8 @@ int lnf_mem_fill_buf(lnf_fieldlist_t *fld, lnf_rec_t *rec, char *buf, int pairse
 		char *ckb = (char *)buf + fld->offset;
 
 		/* detect whether we process aggregation in pair mode - means have pair fields */
-		if (pairset != 0) {
+		/* but only for pair items */
+		if (pairset != 0 && __lnf_fld_pair(fld->field, 1) != LNF_FLD_ZERO_ && __lnf_fld_pair(fld->field, 2) != LNF_FLD_ZERO_) {
 			/* get propper field id depend on field set -  1 - first field, 2 - second field */
 			field = __lnf_fld_pair(fld->field, pairset);
 		} else {
@@ -466,7 +542,11 @@ int lnf_mem_fill_buf(lnf_fieldlist_t *fld, lnf_rec_t *rec, char *buf, int pairse
 			} else {
 				lnf_clear_bits_v6((uint64_t *)ckb, fld->numbits6);
 			}
+		} else if (fld->type == LNF_UINT64 && fld->numbits > 0) { 
+			/* align time fields */
+			lnf_align_uint64((uint64_t *)ckb, fld->numbits);
 		}
+
 		keysize += fld->size;
 		fld = fld->next;
 	}
@@ -555,6 +635,9 @@ int lnf_mem_sort_callback(char *key1, char *val1, char *key2, char *val2, void *
 		case LNF_UINT8: 
 			ret = *(uint8_t *)i1 < *(uint8_t *)i2; 
 			break;
+		case LNF_DOUBLE: 
+			ret = *(LNF_DOUBLE_T *)i1 < *(LNF_DOUBLE_T *)i2; 
+			break;
 		case LNF_ADDR:
 			ret = (memcmp(i1, i2, sizeof(lnf_ip_t)) > 0); 
 			break;
@@ -696,17 +779,28 @@ int lnf_mem_write(lnf_mem_t *lnf_mem, lnf_rec_t *rec) {
 
 	/* insert record for pair set 2 if the statistics mode is enabled */
 	if (pairset != 0) {
+		char keybuf2[LNF_MAX_KEY_LEN]; 
+
 		pairset = 2;
 
 		/* build key for pairset 2 */
-		lnf_mem_fill_buf(lnf_mem->key_list, rec, keybuf, pairset);
+		lnf_mem_fill_buf(lnf_mem->key_list, rec, keybuf2, pairset);
+
+		if (lnf_mem->nfdump_comp_statistics) {
+			if (memcmp(keybuf, keybuf2, lnf_mem->key_len) == 0) {
+				goto END_PAIR;
+			}
+		}
 
 		/* insert record */
-		if (hash_table_insert_hash(&lnf_mem->hash_table[*id], keybuf, valbuf) == NULL) {
+		if (hash_table_insert_hash(&lnf_mem->hash_table[*id], keybuf2, valbuf) == NULL) {
 			return LNF_ERR_NOMEM;
 		} 
 	}
 
+END_PAIR:
+
+
 	return LNF_OK;
 }
 
@@ -787,6 +881,27 @@ int lnf_mem_merge_threads(lnf_mem_t *lnf_mem) {
 #endif	/* ifdef LNF_THREADS */
 }
 
+/* walk via all record and recalculate the value of the calculated fields */
+void lnf_mem_upd_calc_fields(lnf_mem_t *lnf_mem) {
+
+	char *key;
+	char *val;
+
+	lnf_mem_cursor_t *cursor;
+	
+	cursor = (lnf_mem_cursor_t *)hash_table_first(&lnf_mem->hash_table[0]);
+
+	while ( (cursor != NULL ) ) { 
+		hash_table_fetch(&lnf_mem->hash_table[0], (void *)cursor, &key, &val);
+
+    	lnf_mem_fill_rec(lnf_mem->val_list, val, lnf_mem->lnf_rec);
+    	lnf_mem_fill_rec(lnf_mem->key_list, key, lnf_mem->lnf_rec);
+		lnf_mem_fill_buf(lnf_mem->val_list, lnf_mem->lnf_rec, val, 0);
+
+		cursor = (lnf_mem_cursor_t *)hash_table_next(&lnf_mem->hash_table[0], (void *)cursor);
+	}
+}
+
 /* set cursot to first record from memory heap */
 /* used later by lnf_mem_read and lnf_mem_read_raw */
 int lnf_mem_first_c(lnf_mem_t *lnf_mem, lnf_mem_cursor_t **cursor) {
@@ -796,7 +911,13 @@ int lnf_mem_first_c(lnf_mem_t *lnf_mem, lnf_mem_cursor_t **cursor) {
 		return LNF_EOF;
 	}
 
-	if (!lnf_mem->sorted) {
+	if (!lnf_mem->sorted && lnf_mem->sort_field != LNF_FLD_ZERO_) {
+
+		/* if the sorted item is calculated we have to update calculated item first */
+		if (__lnf_fld_calc_dep(lnf_mem->sort_field, 0) != LNF_FLD_ZERO_ ) {
+			lnf_mem_upd_calc_fields(lnf_mem);
+		}
+
 		//hash_table_sort_heap(&lnf_mem->hash_table[0]);
 		hash_table_sort(&lnf_mem->hash_table[0]);
 		lnf_mem->sorted = 1;
@@ -981,6 +1102,10 @@ void lnf_mem_free(lnf_mem_t *lnf_mem) {
 		lnf_filedlist_free(lnf_mem->val_list);
 	}
 
+#ifdef LNF_THREADS
+	pthread_key_delete(lnf_mem->thread_id_key);
+#endif
+
 	free(lnf_mem);
 
 }
