@@ -137,7 +137,7 @@ extension_descriptor_t extension_descriptor[] = {
 	{ 0,	0,	0, 0,	NULL }
 };
 
-uint32_t Max_num_extensions;
+uint32_t Max_num_extensions = 0;
 
 void FixExtensionMap(extension_map_t *map);
 
@@ -154,10 +154,14 @@ int i;
 		list->last_map = &list->map_list;
 	}
 
-	Max_num_extensions = 0;
-	i = 1;
-	while ( extension_descriptor[i++].id ) {
-		Max_num_extensions++;
+	/* modifie by libnf patch */
+	/* Max_num_extensions can be set onluy once */
+//	Max_num_extensions = 0;
+	if (Max_num_extensions == 0) {
+		i = 1;
+		while ( extension_descriptor[i++].id ) {
+			Max_num_extensions++;
+		}
 	}
 #ifdef DEVEL
 	i = 1;
